@@ -1,7 +1,7 @@
-# $Id: UserAgent.pm,v 1.51 1997/12/02 13:22:53 aas Exp $
+# $Id: UserAgent.pm,v 1.58 1998/01/21 12:53:21 aas Exp $
 
 package LWP::UserAgent;
-
+use strict;
 
 =head1 NAME
 
@@ -79,20 +79,6 @@ content in the request object.  This subroutine should return the
 content (possibly in pieces) when called.  It should return an empty
 string when there is no more content.
 
-The user of this module can finetune timeouts and error handling by
-calling the use_alarm() and use_eval() methods.
-
-By default the library uses alarm() to implement timeouts, dying if
-the timeout occurs. If this is not the prefered behaviour or it
-interferes with other parts of the application one can disable the use
-alarms. When alarms are disabled timeouts can still occur for example
-when reading data, but other cases like name lookups etc will not be
-timed out by the library itself.
-
-The library catches errors (such as internal errors and timeouts) and
-present them as HTTP error responses. Alternatively one can switch off
-this behaviour, and let the application handle dies.
-
 =head1 METHODS
 
 The following methods are available:
@@ -102,9 +88,12 @@ The following methods are available:
 =cut
 
 
+use vars qw(@ISA $VERSION);
 
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.58 $ =~ /(\d+)\.(\d+)/);
+
 
 require URI::URL;
 require HTTP::Request;
@@ -117,7 +106,6 @@ use LWP::Debug ();
 use LWP::Protocol ();
 
 use Carp ();
-use Config ();
 
 use AutoLoader ();
 *AUTOLOAD = \&AutoLoader::AUTOLOAD;  # import the AUTOLOAD method
@@ -146,9 +134,6 @@ sub new
 		'proxy'       => undef,
 		'cookie_jar'  => undef,
 		'use_eval'    => 1,
-		'use_alarm'   => ($Config::Config{d_alarm} ?
-				  $Config::Config{d_alarm} eq 'define' :
-				  0),
                 'parse_head'  => 1,
                 'max_size'    => undef,
 		'no_proxy'    => [],
@@ -204,37 +189,26 @@ sub simple_request
 	$protocol = LWP::Protocol::create($scheme);
     };
     if ($@) {
-	$@ =~ s/\s+at\s+\S+\s+line\s+\d+//;  # remove file/line number
+	$@ =~ s/\s+at\s+\S+\s+line\s+\d+\.?\s*//;  # remove file/line number
 	return HTTP::Response->new(&HTTP::Status::RC_NOT_IMPLEMENTED, $@)
     }
 
     # Extract fields that will be used below
     my ($agent, $from, $timeout, $cookie_jar,
-        $use_alarm, $use_eval, $parse_head, $max_size) =
+        $use_eval, $parse_head, $max_size) =
       @{$self}{qw(agent from timeout cookie_jar
-                  use_alarm use_eval parse_head max_size)};
+                  use_eval parse_head max_size)};
 
     # Set User-Agent and From headers if they are defined
     $request->header('User-Agent' => $agent) if $agent;
     $request->header('From' => $from) if $from;
     $cookie_jar->add_cookie_header($request) if $cookie_jar;
 
-    # Inform the protocol if we need to use alarm() and parse_head()
-    $protocol->use_alarm($use_alarm);
+    # Transfer some attributes to the protocol object
     $protocol->parse_head($parse_head);
     $protocol->max_size($max_size);
     
-    # If we use alarm() we need to register a signal handler
-    # and start the timeout
-    if ($use_alarm) {
-	$SIG{'ALRM'} = sub {
-	    LWP::Debug::trace('timeout');
-	    die 'Timeout';
-	};
-	$protocol->timeout($timeout);
-	alarm($timeout);
-    }
-
+    my $response;
     if ($use_eval) {
 	# we eval, and turn dies into responses below
 	eval {
@@ -242,20 +216,16 @@ sub simple_request
 					   $arg, $size, $timeout);
 	};
 	if ($@) {
-	    if ($@ =~ /^timeout/i) {
-		$response = HTTP::Response->new(&HTTP::Status::RC_REQUEST_TIMEOUT, 'User-agent timeout');
-	    } else {
-		$@ =~ s/\s+at\s+\S+\s+line\s+\d+\s*//;  # remove file/line number
-		$response = HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR, $@);
-	    }
+	    $@ =~ s/\s+at\s+\S+\s+line\s+\d+\.?\s*//;
+	    $response =
+	      HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+				  $@);
 	}
     } else {
-	# user has to handle any dies, usually timeouts
 	$response = $protocol->request($request, $proxy,
 				       $arg, $size, $timeout);
 	# XXX: Should we die unless $response->is_success ???
     }
-    alarm(0) if ($use_alarm); # no more timeout
 
     $response->request($request);  # record request for reference
     $cookie_jar->extract_cookies($response) if $cookie_jar;
@@ -344,6 +314,7 @@ sub request
 	my $class = "LWP::Authen::\u$scheme";
 	$class =~ s/-/_/g;
 	
+	no strict 'refs';
 	unless (defined %{"$class\::"}) {
 	    # try to load it
 	    eval "require $class";
@@ -467,18 +438,6 @@ Get/set the I<HTTP::Cookies> object to use.  The default is to have no
 cookie_jar, i.e. never automatically add "Cookie" headers to the
 requests.
 
-=item $ua->use_alarm([$boolean])
-
-Get/set a value indicating wether to use alarm() when implementing
-timeouts.  The default is TRUE, if your system supports it.  You can
-disable it if it interfers with other uses of alarm in your application.
-
-=item $ua->use_eval([$boolean])
-
-Get/set a value indicating wether to handle internal errors internally
-by trapping with eval.  The default is TRUE, i.e. the $ua->request()
-will never die.
-
 =item $ua->parse_head([$boolean])
 
 Get/set a value indicating wether we should initialize response
@@ -498,11 +457,18 @@ sub timeout    { shift->_elem('timeout',   @_); }
 sub agent      { shift->_elem('agent',     @_); }
 sub from       { shift->_elem('from',      @_); }
 sub cookie_jar { shift->_elem('cookie_jar',@_); }
-sub use_alarm  { shift->_elem('use_alarm', @_); }
-sub use_eval   { shift->_elem('use_eval',  @_); }
 sub parse_head { shift->_elem('parse_head',@_); }
 sub max_size   { shift->_elem('max_size',  @_); }
 
+# depreciated
+sub use_eval   { shift->_elem('use_eval',  @_); }
+sub use_alarm
+{
+    Carp::carp("LWP::UserAgent->use_alarm(BOOL) is a no-op")
+	if @_ > 1 && $^W;
+    "";
+}
+
 
 # Declarations of AutoLoaded methods
 sub clone;
@@ -665,6 +631,7 @@ envirionment variables.
 
 sub env_proxy {
     my ($self) = @_;
+    my($k,$v);
     while(($k, $v) = each %ENV) {
 	$k = lc($k);
 	next unless $k =~ /^(.*)_proxy$/;
